@@ -80,6 +80,7 @@ typedef enum {
 	SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR,
 	SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR,
 	SLURMCTLD_COMMUNICATIONS_BACKOFF,
+	SLURMCTLD_COMMUNICATIONS_HARD_DROP,
 
 	/* _info.c/communication layer RESPONSE_SLURM_RC message codes */
 	SLURM_NO_CHANGE_IN_DATA =			1900,
@@ -259,6 +260,10 @@ typedef enum {
 	ESLURM_INVALID_EXTRA,
 	ESLURM_JOB_SIGNAL_FAILED,
 	ESLURM_SIGNAL_JOBS_INVALID,
+	ESLURM_RES_CORES_PER_GPU_UNIQUE,
+	ESLURM_RES_CORES_PER_GPU_TOPO,
+	ESLURM_RES_CORES_PER_GPU_NO,
+	ESLURM_MAX_POWERED_NODES,
 
 	/* SPANK errors */
 	ESPANK_ERROR = 					3000,
@@ -299,6 +304,7 @@ typedef enum {
 	ESLURMD_CONTAINER_RUNTIME_INVALID,
 	ESLURMD_CPU_BIND_ERROR,
 	ESLURMD_CPU_LAYOUT_ERROR,
+	ESLURMD_TOO_MANY_RPCS,
 
 	/* socket specific Slurm communications error */
 	ESLURM_PROTOCOL_INCOMPLETE_PACKET = 5003,
@@ -325,6 +331,7 @@ typedef enum {
 	ESLURM_NO_REMOVE_DEFAULT_ACCOUNT,
 	ESLURM_BAD_SQL,
 	ESLURM_NO_REMOVE_DEFAULT_QOS,
+	ESLURM_COORD_NO_INCREASE_JOB_LIMIT,
 
 	/* Federation Errors */
 	ESLURM_FED_CLUSTER_MAX_CNT              = 7100,
@@ -371,6 +378,8 @@ typedef enum {
 	ESLURM_DATA_AMBIGUOUS_QUERY,
 	ESLURM_DATA_PARSE_NOTHING,
 	ESLURM_DATA_INVALID_PARSER,
+	ESLURM_DATA_PARSING_DEPTH,
+	ESLURM_DATA_PARSER_INVALID_STATE,
 
 	/* container errors */
 	ESLURM_CONTAINER_NOT_CONFIGURED = 10000,
