@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2010 Lawrence Livermore National Security.
- *  Portions Copyright (C) 2010 SchedMD <https://www.schedmd.com>.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Morris Jette <jette1@llnl.gov> and Kevin Tew <tew1@llnl.gov>.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -118,7 +118,7 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 	partition_info_t *p = NULL;
 	struct records {
 	  char *rec;
-	  hostlist_t hostlist;
+	  hostlist_t *hostlist;
 	  struct records *next;
 	} *rp = NULL;
 	struct records *crp;
@@ -322,6 +322,9 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 		if (p[i].flags & PART_FLAG_EXCLUSIVE_USER)
 			fprintf(fp, " ExclusiveUser=YES");
 
+		if (p[i].flags & PART_FLAG_EXCLUSIVE_TOPO)
+			fprintf(fp, " ExclusiveTopo=YES");
+
 		if (p[i].grace_time)
 			fprintf(fp, " GraceTime=%u", p[i].grace_time);
 
@@ -445,7 +448,7 @@ void slurm_write_ctl_conf ( slurm_ctl_conf_info_msg_t * slurm_ctl_conf_ptr,
 
 static void _print_config_plugin_params_list(FILE* out, List l, char *title)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	config_plugin_params_t *p;
 
 	if (!l || !list_count(l))
@@ -472,12 +475,8 @@ void slurm_print_ctl_conf ( FILE* out,
 	char time_str[32], tmp_str[256];
 	void *ret_list = NULL;
 	char *select_title = "Select Plugin Configuration";
-	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
 	char *tmp2_str = NULL;
 
-	if (cluster_flags & CLUSTER_FLAG_CRAY)
-		select_title = "\nCray configuration\n";
-
 	if (slurm_ctl_conf_ptr == NULL)
 		return;
 
@@ -498,8 +497,6 @@ void slurm_print_ctl_conf ( FILE* out,
 	slurm_print_key_pairs(out, slurm_ctl_conf_ptr->cgroup_conf,
 			      "\nCgroup Support Configuration:\n");
 
-	slurm_print_key_pairs(out, slurm_ctl_conf_ptr->ext_sensors_conf,
-			      "\nExternal Sensors Configuration:\n");
 	slurm_print_key_pairs(out, slurm_ctl_conf_ptr->mpi_conf,
 			      "\nMPI Plugins Configuration:\n");
 
@@ -519,6 +516,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	char tmp_str[256];
 	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
 	int i;
+	bool format_stderr = false;
 
 	if ( slurm_ctl_conf_ptr == NULL )
 		return NULL;
@@ -579,18 +577,21 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("AccountingStoreFlags");
-	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJC)
+	if (slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_SJC)
 		xstrfmtcat(key_pair->value, "%sjob_comment",
 			   key_pair->value ? "," : "");
-	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJE)
+	if (slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_SJE)
 		xstrfmtcat(key_pair->value, "%sjob_env",
 			   key_pair->value ? "," : "");
-	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJX)
+	if (slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_SJX)
 		xstrfmtcat(key_pair->value, "%sjob_extra",
 			   key_pair->value ? "," : "");
-	if (slurm_ctl_conf_ptr->conf_flags & CTL_CONF_SJS)
+	if (slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_SJS)
 		xstrfmtcat(key_pair->value, "%sjob_script",
 			   key_pair->value ? "," : "");
+	if (slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_NO_STDIO)
+		xstrfmtcat(key_pair->value, "%sno_stdio",
+			   key_pair->value ? "," : "");
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -625,7 +626,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("AllowSpecResourcesUsage");
 	key_pair->value = xstrdup(
-		(slurm_ctl_conf_ptr->conf_flags & CTL_CONF_ASRU) ?
+		(slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_ASRU) ?
 		"Yes" : "No");
 	list_append(ret_list, key_pair);
 
@@ -700,11 +701,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("CoreSpecPlugin");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->core_spec_plugin);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("CpuFreqDef");
 	cpu_freq_to_string(tmp_str, sizeof(tmp_str),
@@ -759,7 +755,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	list_append(ret_list, key_pair);
 	key_pair->name = xstrdup("DisableRootJobs");
 	key_pair->value = xstrdup(
-		(slurm_ctl_conf_ptr->conf_flags & CTL_CONF_DRJ) ? "Yes" : "No");
+		(slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_DRJ) ? "Yes" : "No");
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("EioTimeout");
@@ -775,10 +771,12 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 		parse_part_enforce_type_2str(
 			slurm_ctl_conf_ptr->enforce_part_limits));
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("Epilog");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->epilog);
-	list_append(ret_list, key_pair);
+	for (int i = 0; i < slurm_ctl_conf_ptr->epilog_cnt; i++) {
+		char *key = xstrdup_printf("Epilog[%d]", i);
+		add_key_pair(ret_list, key, "%s",
+			     slurm_ctl_conf_ptr->epilog[i]);
+		xfree(key);
+	}
 
 	snprintf(tmp_str, sizeof(tmp_str), "%u usec",
 		 slurm_ctl_conf_ptr->epilog_msg_time);
@@ -787,22 +785,12 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("EpilogSlurmctld");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->epilog_slurmctld);
-	list_append(ret_list, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ExtSensorsType");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->ext_sensors_type);
-	list_append(ret_list, key_pair);
-
-	snprintf(tmp_str, sizeof(tmp_str), "%u sec",
-		 slurm_ctl_conf_ptr->ext_sensors_freq);
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("ExtSensorsFreq");
-	key_pair->value = xstrdup(tmp_str);
-	list_append(ret_list, key_pair);
+	for (int i = 0; i < slurm_ctl_conf_ptr->epilog_slurmctld_cnt; i++) {
+		char *key = xstrdup_printf("EpilogSlurmctld[%d]", i);
+		add_key_pair(ret_list, key, "%s",
+			     slurm_ctl_conf_ptr->epilog_slurmctld[i]);
+		xfree(key);
+	}
 
 	if (xstrcmp(slurm_ctl_conf_ptr->priority_type, "priority/basic")) {
 		snprintf(tmp_str, sizeof(tmp_str), "%u",
@@ -871,6 +859,9 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 		list_append(ret_list, key_pair);
 	}
 
+	add_key_pair(ret_list, "HashPlugin", "%s",
+		     slurm_ctl_conf_ptr->hash_plugin);
+
 	snprintf(tmp_str, sizeof(tmp_str), "%u sec",
 		 slurm_ctl_conf_ptr->health_check_interval);
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -953,18 +944,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->job_container_plugin);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("JobCredentialPrivateKey");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->
-				  job_credential_private_key);
-	list_append(ret_list, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("JobCredentialPublicCertificate");
-	key_pair->value = xstrdup(
-		slurm_ctl_conf_ptr->job_credential_public_certificate);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("JobDefaults");
 	key_pair->value =
@@ -1016,6 +995,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("LogTimeFormat");
+	if (slurm_ctl_conf_ptr->log_fmt & LOG_FMT_FORMAT_STDERR) {
+		format_stderr = true;
+		slurm_ctl_conf_ptr->log_fmt &= (~LOG_FMT_FORMAT_STDERR);
+	}
+
 	if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_ISO8601_MS)
 		key_pair->value = xstrdup("iso8601_ms");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_ISO8601)
@@ -1032,6 +1016,11 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 		key_pair->value = xstrdup("short");
 	else if (slurm_ctl_conf_ptr->log_fmt == LOG_FMT_THREAD_ID)
 		key_pair->value = xstrdup("thread_id");
+
+	if (format_stderr) {
+		xstrcat(key_pair->value, ",format_stderr");
+		slurm_ctl_conf_ptr->log_fmt |= LOG_FMT_FORMAT_STDERR;
+	}
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -1194,16 +1183,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->plugstack);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("PowerParameters");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->power_parameters);
-	list_append(ret_list, key_pair);
-
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("PowerPlugin");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->power_plugin);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("PreemptMode");
 	key_pair->value = xstrdup(preempt_mode_string(slurm_ctl_conf_ptr->
@@ -1370,10 +1349,12 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->proctrack_type);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("Prolog");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->prolog);
-	list_append(ret_list, key_pair);
+	for (int i = 0; i < slurm_ctl_conf_ptr->prolog_cnt; i++) {
+		char *key = xstrdup_printf("Prolog[%d]", i);
+		add_key_pair(ret_list, key, "%s",
+			     slurm_ctl_conf_ptr->prolog[i]);
+		xfree(key);
+	}
 
 	snprintf(tmp_str, sizeof(tmp_str), "%u",
 		 slurm_ctl_conf_ptr->prolog_epilog_timeout);
@@ -1382,10 +1363,12 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("PrologSlurmctld");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->prolog_slurmctld);
-	list_append(ret_list, key_pair);
+	for (int i = 0; i < slurm_ctl_conf_ptr->prolog_slurmctld_cnt; i++) {
+		char *key = xstrdup_printf("PrologSlurmctld[%d]", i);
+		add_key_pair(ret_list, key, "%s",
+			     slurm_ctl_conf_ptr->prolog_slurmctld[i]);
+		xfree(key);
+	}
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("PrologFlags");
@@ -1482,11 +1465,6 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
-	key_pair = xmalloc(sizeof(config_key_pair_t));
-	key_pair->name = xstrdup("RoutePlugin");
-	key_pair->value = xstrdup(slurm_ctl_conf_ptr->route_plugin);
-	list_append(ret_list, key_pair);
-
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("SchedulerParameters");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->sched_params);
@@ -1809,6 +1787,10 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 					 slurm_ctl_conf_ptr->tcp_timeout);
 	list_append(ret_list, key_pair);
 
+	add_key_pair(ret_list, "TLSParameters", "%s",
+		     slurm_ctl_conf_ptr->tls_params);
+	add_key_pair(ret_list, "TLSType", "%s", slurm_ctl_conf_ptr->tls_type);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("TmpFS");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->tmp_fs);
@@ -1827,7 +1809,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("TrackWCKey");
 	key_pair->value = xstrdup(
-		(slurm_ctl_conf_ptr->conf_flags & CTL_CONF_WCKEY) ?
+		(slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_WCKEY) ?
 		"Yes" : "No");
 	list_append(ret_list, key_pair);
 
@@ -1841,7 +1823,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("UsePam");
 	key_pair->value = xstrdup(
-		(slurm_ctl_conf_ptr->conf_flags & CTL_CONF_PAM) ? "Yes" : "No");
+		(slurm_ctl_conf_ptr->conf_flags & CONF_FLAG_PAM) ? "Yes" : "No");
 	list_append(ret_list, key_pair);
 
 	key_pair = xmalloc(sizeof(config_key_pair_t));
@@ -1875,7 +1857,7 @@ extern void *slurm_ctl_conf_2_key_pairs(slurm_conf_t *slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->x11_params);
 	list_append(ret_list, key_pair);
 
-	return (void *)ret_list;
+	return ret_list;
 }
 
 /*
@@ -1944,8 +1926,23 @@ slurm_load_slurmd_status(slurmd_status_t **slurmd_status_ptr)
 
 	if (cluster_flags & CLUSTER_FLAG_MULTSD) {
 		if ((this_addr = getenv("SLURMD_NODENAME"))) {
-			slurm_conf_get_addr(this_addr, &req_msg.address,
-					    req_msg.flags);
+			if (slurm_conf_get_addr(this_addr, &req_msg.address,
+						req_msg.flags)) {
+				/*
+				 * The node isn't in the conf, see if the
+				 * controller has an address for it.
+				 */
+				slurm_node_alias_addrs_t *alias_addrs;
+				if (!slurm_get_node_alias_addrs(this_addr,
+								&alias_addrs)) {
+					add_remote_nodes_to_conf_tbls(
+						alias_addrs->node_list,
+						alias_addrs->node_addrs);
+				}
+				slurm_free_node_alias_addrs(alias_addrs);
+				slurm_conf_get_addr(this_addr, &req_msg.address,
+						    req_msg.flags);
+			}
 		} else {
 			this_addr = "localhost";
 			slurm_set_addr(&req_msg.address, slurm_conf.slurmd_port,
@@ -2067,7 +2064,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 	config_key_pair_t *key_pair;
 	char *temp = NULL;
 	List config_list = (List)key_pairs;
-	ListIterator iter = NULL;
+	list_itr_t *iter = NULL;
 	/* define lists of specific configuration sections */
 	List other_list;
 	List control_list;
@@ -2207,9 +2204,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 		    !xstrcasecmp(key_pair->name, "AcctGatherNodeFreq") ||
 		    !xstrcasecmp(key_pair->name, "AcctGatherProfileType") ||
 		    !xstrcasecmp(key_pair->name, "JobAcctGatherFrequency") ||
-		    !xstrcasecmp(key_pair->name, "JobAcctGatherType") ||
-		    !xstrcasecmp(key_pair->name, "ExtSensorsType") ||
-		    !xstrcasecmp(key_pair->name, "ExtSensorsFreq")) {
+		    !xstrcasecmp(key_pair->name, "JobAcctGatherType")) {
 			list_append(accounting_list, temp);
 			continue;
 		}
@@ -2391,7 +2386,7 @@ static void _write_key_pairs(FILE* out, void *key_pairs)
 extern void slurm_print_key_pairs(FILE* out, void *key_pairs, char *title)
 {
 	List config_list = (List)key_pairs;
-	ListIterator iter = NULL;
+	list_itr_t *iter = NULL;
 	config_key_pair_t *key_pair;
 
 	if (!config_list || !list_count(config_list))
